#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeSnippet = void 0;
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = __importDefault(require("fs"));
const recursive_readdir_1 = __importDefault(require("recursive-readdir"));
const utils_1 = require("./utils");
class CodeSnippet {
    snippet;
    config;
    prompt;
    path;
    args;
    constructor() {
        this.snippet = new utils_1.UtilSnippet();
        this.prompt = new utils_1.UtilPrompt();
        this.config = new utils_1.UtilConfig().generate();
        this.path = new utils_1.UtilPath();
        this.args = {
            snippet: '',
            name: '',
            path: '',
            isFlat: false,
        };
    }
    generateUnformattedSnippet(pathToSnippet) {
        const unformattedSnippet = fs_1.default.readFileSync(pathToSnippet, 'utf-8');
        return JSON.stringify(unformattedSnippet);
    }
    generateFormattedSnippet(unformattedSnippet) {
        const formattedSnippet = this.snippet.replace(unformattedSnippet, this.args.name);
        return JSON.parse(formattedSnippet);
    }
    createSnippet(options) {
        const mkdir = this.args.isFlat ? options.pathFlatSnippetFile : options.pathSnippetFile;
        const writeFile = this.args.isFlat ? options.pathFlatWriteSnippetFile : options.pathWriteSnippetFile;
        fs_1.default.mkdirSync(mkdir, { recursive: true });
        fs_1.default.writeFileSync(writeFile, options.formattedSnippet);
        // eslint-disable-next-line no-console
        console.log(`${chalk_1.default.green(`√`)} ${chalk_1.default.gray(`${options.snippetFileName}`)}`);
    }
    async start() {
        try {
            const args = await this.prompt.getArgs();
            this.args = { ...this.args, ...args };
            const config = this.config.find(({ snippet }) => snippet === this.args.snippet);
            if (!config)
                throw new Error('config');
            if (this.args.snippet === '')
                throw new Error(`specify the required argument snippet=example`);
            if (this.args.name === '')
                throw new Error(`specify the required argument name=example`);
            if (this.args.path === '')
                throw new Error(`specify the required argument path=example`);
            (0, recursive_readdir_1.default)(config.files, (error, paths) => {
                if (error)
                    throw new Error(error.message);
                paths.forEach((pathToSnippet) => {
                    const unformattedSnippet = this.generateUnformattedSnippet(pathToSnippet);
                    const formattedSnippet = this.generateFormattedSnippet(unformattedSnippet);
                    const path = this.path.generate({
                        pathToSnippet,
                        config,
                        name: this.args.name,
                        path: this.args.path,
                    });
                    if (fs_1.default.existsSync(path.pathWriteSnippetFile))
                        throw new Error(this.args.name);
                    this.createSnippet({ ...path, formattedSnippet });
                });
            });
        }
        catch (e) {
            // eslint-disable-next-line no-console
            console.log(chalk_1.default.red(e));
        }
    }
}
exports.CodeSnippet = CodeSnippet;
const codeSnippet = new CodeSnippet();
codeSnippet.start();
//# sourceMappingURL=index.js.map